### This is the commented R Code used to perform 
### the analysis of Hurricane Data in Cabras, Castellanos and Gamerman "A default Bayesian approach for regression on extremes".

rm(list=ls(all=TRUE))
setwd("~/Dropbox/Articulos/extremes/Rcode-data-files")
#setwd("damage/")
library(mnormt)
library(foreign)
library(MASS)
library(POT)

set.seed(17)

# Simulations from a Generalized Pareto Distributions
# in xi, sigma parameterization
pareto.simulations<-function(n, xi,  sigma,  u=0){
	res=rep(NA,n)
	exp.case=abs(xi)<1e-10
	nexp.case=!exp.case
    res[exp.case]=rexp(sum(exp.case),1/sigma[exp.case])
    res[nexp.case]=u+(sigma[nexp.case]/xi[nexp.case])*((1-runif(sum(nexp.case)))^(-xi[nexp.case])-1)
    return(res)
}

# Change of parameterization from xi.nu to xi.sigma
# return xi.sigma parameterization
xi.nu.to.xi.sigma=function(par) cbind(par[,1],par[,2]/(1+par[,1]))

# Simulations from a GPD process with z,x covariates
sim.gpd.cov=function(n,beta,gamma,z,x){
	xi.lp=z%*%gamma # Linear predictor for xi
	nu.lp=x%*%beta # Linear predictor for nu
	nu=exp(nu.lp)
	xi=exp(xi.lp)-1
	xi.sigma= xi.nu.to.xi.sigma(cbind(xi,nu))
	y=pareto.simulations(n,xi.sigma[,1],xi.sigma[,2])
	return(y)
}



datadam=read.table("hurricane.csv",sep=";",header=TRUE)
datadam$Damage=datadam$Damage/1000 # Data in Billions of 95 US $

# threshold in billions
u=0.01

# Construction of Design Matrix for Poisson-Process intensity
ws=datadam[,-2]
ws=unique(ws)
tmp=1:nrow(ws)

##To adjust the model with interactions to the counting part, model e in the paper
#ws=lm(tmp~ws$enso+cos(ws[,1])*ws$enso+sin(ws[,1])*ws$enso,x=TRUE)$x 
#colnames(ws)=c("I","niño","neutral","cos","sin","cos.O","cos.N","sin.O","sin.N")

ws=lm(tmp~ws$enso+cos(ws[,1])+sin(ws[,1]),x=TRUE)$x # Model d
colnames(ws)=c("I","niño","neutral","cos","sin")

#ws=lm(tmp~1+cos(ws[,1])+sin(ws[,1]),x=TRUE)$x # Model c
#colnames(ws)=c("I","cos","sin")

#ws=lm(tmp~ws$enso,x=TRUE)$x # Model b
#colnames(ws)=c("I","niño","neutral")
#ws=unique(ws) # only in Model b

# Index of Phases per year
year.phase=unique(cbind(datadam$Year,datadam$enso))
indi.ninya=(year.phase[,2]==1)
indi.ninyo=(year.phase[,2]==2)
indi.neutral=(year.phase[,2]==3)

# Exceedances
datadam$Damage=datadam$Damage-u
datadam=datadam[datadam$Damage>0,]
enso=datadam$enso
table(datadam$enso)

tmp=table(datadam$Year)
ex.years=as.numeric(names(tmp))
int.years=c(1925:1995)
count.ex=rep(0,length(int.years))
names(count.ex)=int.years
for(i in int.years) if(any(ex.years==i)) count.ex[int.years==i]=tmp[ex.years==i]

# count.ex=table(datadam$enso) # For ENSO only



n=nrow(datadam)  # Sample size
y=datadam$Damage # Exceedances

# Figure 6 - Data
plot(datadam$Year,y,type="p",main="Hurricanes Damage (1995 US$)",xlab="Time period",ylab="Billion of $",pch=19+as.numeric(enso),col=1,cex.lab=1.2,cex.axis=1.2,cex.main=1.2,cex=1.2)
legend(1940,50,c("niña","niño","neutral"),pch=c(20,21,22),cex=1.2)

# Definition of desing matrices x and z, for nu and xi, respectively:
mylm=lm(y~enso,x=TRUE)
x=z=mylm$x
colnames(x)=colnames(z)=c("I","niño","neutral")

p=q=ncol(x)
r=ncol(ws)
r
p
q
n

##MCMC simulations
nrep=20000
burn.in=10000
totsim=burn.in+nrep

# log posterior kernel
l.zx=function(y,z,x,beta,gamma,n){
	xi=exp(z%*%gamma)-1
    if(any(xi<(-0.5))) return(-Inf)
	nu=exp(x%*%beta)
	if(any((1+y*(xi^2+xi)/nu)<=0)) return(-Inf)
	return(sum(log((1+xi))-log(nu)-(1/xi+1)*log(1+y*(xi^2+xi)/nu)))
	}
kp=function(theta) l.zx(y,z,x,beta=theta[1:p],gamma=theta[(p+1):(p+q)],n)


# Posterior mode
init.point=rep(0.1,p+q)
tmp=optim(par=init.point,kp,control=list(fnscale=-1,maxit=10000),hessian=TRUE,method=c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN")[1])
tmp$convergence
mode=c(tmp$par)
mode

# Setup of MCMC for the GPD part of posterior
varlabel=c(paste("beta[",1:p,"]",sep=""),paste("gamma[",1:q,"]",sep=""))
theta=array(NA,dim=c(burn.in+nrep,p+q))
theta.star=rep(NA,p+q)
colnames(theta)=varlabel
acc=rep(0,p+q)
fv=theta.star=rep(NA,p+q)
theta.star=lr=sampord=array(NA,dim=c(totsim,p+q))
theta[1,]=mode

# Standard Deviation of the Jumping distribution
#sigma.jump=c(0.5,1,0.8,0.3,0.5,0.4)

# Automatically, one can use, as initial values for the variance
# of proposal, the diagonal elements of the inverse of Hessian 
# matrix of log-posterior kernel, calculated at posterior mode
# in order to reach a reasonable acceptance rate we multiply such elements by a factor k (here k is 2).
sigma.jump=sqrt(abs(diag(solve(tmp$hessian))))*2


# Sample of Candidates
for(i in 2:totsim){
	lr[i,]=log(runif(p+q))
	sampord[i,]=sample(p+q,size=p+q)
	theta.star[i,]=rnorm(p+q,mean=0,sd=sigma.jump)
	}

# The MCMC and the Acceptance Rate (acc)
for(i in 2:totsim){
	theta.star[i,]=theta.star[i,]+theta[i-1,]
	theta[i,]=theta[i-1,]
	kpold=kp(theta[i-1,])
	for(j in sampord[i,]){fv[-j]=theta[i,-j];fv[j]=theta.star[i,j];if(lr[i,j]<(kp(fv)-kpold)) theta[i,j]=theta.star[i,j]}
	acc=acc+(theta[i,]!=theta[i-1,])
	cat("i=",i,"AR=",round(acc/i,1)*10,"\n")
}
theta=theta[-(1:burn.in),]


# MCMC diagnostics
par(mfrow=c(3,3)) ##Beta parameters, parameters associated with nu:
for(j in 1:p){
	plot.ts(theta[,j],main=varlabel[j])
    acf(theta[,j],main=varlabel[j])
	hist(theta[,j],main=varlabel[j])
}
par(mfrow=c(3,3)) ##Gamma parameters, parameters associated with xi:
for(j in (p+1):(p+q)){ 
	plot.ts(theta[,j],main=varlabel[j])
  acf(theta[,j],main=varlabel[j])
	hist(theta[,j],main=varlabel[j])
}

####################################################
##Probabilities in Table 3: Pr(Z>z |y,x):

# Predictive of Exceedances for niña: 1,0,0
ypred=apply(theta,1,function(param) sim.gpd.cov(1,param[1:p],gamma=param[(p+1):(p+q)],z=c(1,0,0),x=c(1,0,0)))
mean(ypred>(1-u))
mean(ypred>(5-u))
mean(ypred>(10-u))

# Predictive of Exceedances for niño: 1,1,0
ypred=apply(theta,1,function(param) sim.gpd.cov(1,param[1:p],gamma=param[(p+1):(p+q)],z=c(1,1,0),x=c(1,1,0)))
mean(ypred>(1-u))
mean(ypred>(5-u))
mean(ypred>(10-u))


# Predictive of Exceedances for other: 1,0,1
ypred=apply(theta,1,function(param) sim.gpd.cov(1,param[1:p],gamma=param[(p+1):(p+q)],z=c(1,0,1),x=c(1,0,1)))
mean(ypred>(1-u))
mean(ypred>(5-u))
mean(ypred>(10-u))



# LOG-Likelihood for Poisson Process 
deltai=1  #in Years
#deltai=c(sum(indi.ninya),sum(indi.ninyo),sum(indi.neutral)) # For only ENSO covariate 


l.poisson=function(alpha,ws=ws,deltai=deltai) sum(count.ex*c(log((ws%*%alpha)^2)))-sum(deltai*c((ws%*%alpha)^2))
alpha1.init=sqrt(49/22)
alpha2.init=(-2+sqrt(4-4*(alpha1.init^2-29/21)))/2
alpha3.init=(-2+sqrt(4-4*(alpha1.init^2-59/27)))/2


# Posterior MODE
#With interaction between enso and sin, cosin:
init.point=c(alpha1.init, alpha2.init, alpha3.init,1,1)

tmp=optim(par=init.point,l.poisson,control=list(fnscale=-1,maxit=10000),hessian=TRUE,method=c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN")[1],ws=ws, deltai=deltai)
tmp$convergence
mode.pois=c(tmp$par)
mode.pois

# Setup of MCMC for the GPD part of posterior

# Standard Deviation of Proposal Distribution
#With interaction between enso and sin, cosin:
# sigma.jump=(c(0.02,0.03,0.03,0.02,0.02)*12)
# Automatically, one can use, as initial values for the variance
# of proposal, the diagonal elements of the inverse of Hessian 
# matrix of log-posterior kernel, calculated at posterior mode
# in order to reach a reasonable acceptance rate we multiply such elements by a factor k (here k is 2).
sigma.jump=sqrt(abs(diag(solve(tmp$hessian))))*2

varlabel=paste("alpha[",1:r,"]",sep="")
theta.pois=array(NA,dim=c(totsim,r))
colnames(theta.pois)=varlabel
theta.pois[1,]=mode.pois
acc=rep(0,r)
fv=theta.star=rep(NA,r)
theta.star=lr=sampord=array(NA,dim=c(totsim,r))

# The MCMC and the Acceptance Rate (acc)
for(i in 2:totsim){
	lr[i,]=log(runif(r))
	sampord[i,]=sample(r,size=r)
	theta.star[i,]=rnorm(r,mean=0,sd=sigma.jump)
	}
for(i in 2:totsim){
	theta.star[i,]=theta.star[i,]+theta.pois[i-1,]
	theta.pois[i,]=theta.pois[i-1,]
	kpold=l.poisson(theta.pois[i-1,],ws=ws, deltai=deltai)
	for(j in sampord[i,]){fv[-j]=theta.pois[i,-j];fv[j]=theta.star[i,j];if(lr[i,j]<(l.poisson(fv,ws=ws,deltai=deltai)-kpold)) theta.pois[i,j]=theta.star[i,j]}
	acc=acc+(theta.pois[i,]!=theta.pois[i-1,])
	cat("i=",i,"AR=",round(acc/i,1)*10,"\n")
}
theta.pois=theta.pois[-(1:burn.in),]


# MCMC Diagnostic
par(mfrow=c(5,3))
for(j in 1:5){
	plot.ts(theta.pois[,j],main=varlabel[j])
    acf(theta.pois[,j],main=varlabel[j])
	hist(theta.pois[,j],main=varlabel[j])
}


######################## Analyses of the Joint Posterior Distribution ########################
# Posterior Predictive distribution of NU, XI, Lambda and Mean Return Level (mrl) for N-years (N=1,50,70,100) given ENSO and Year

mrl1.enso=array(NA,dim=c(nrep,3,71))
nu.enso=xi.enso=lambda.enso=array(NA,dim=c(nrep,3))
for(i in 1:3){
	for(j in 1:71){
	dmm=cbind(c(1,0,0,cos(1924+j),sin(1924+j)),
		  c(1,1,0,cos(1924+j),sin(1924+j)),
	  	  c(1,0,1,cos(1924+j),sin(1924+j)))
	dm=c(dmm[,i])
	nu.enso[,i]=nu.y=c(exp(dm[1:3]%*%t(theta[,1:p])))
	xi.enso[,i]=xi.y=c(exp(dm[1:3]%*%t(theta[,(p+1):(p+q)]))-1)
	lambda.enso[,i]=lambda.y=c((dm%*%t(theta.pois))^2)
	mrl1.enso[,i,j]=mrl1.y=c(u+(nu.y/(xi.y*(1+xi.y)))*((lambda.y*1)^xi.y-1))
	mrl1.enso[,i,j][mrl1.enso[,i,j]<=0]=0.01
	}
}

# (PLOT) Posterior Distribution of NU and Xi given Enso
fseas=factor(c(rep("niña",nrep),rep("niño",nrep),rep("neutral",nrep)),levels=c("niña","niño","neutral"))
nus=c(nu.enso[,1],nu.enso[,2],nu.enso[,3])
xis=c(xi.enso[,1],xi.enso[,2],xi.enso[,3])
par(mfrow=c(3,2))
hist(nus[fseas=="niña"],xlab=expression(nu),main=expression(paste(pi,"(",nu," |y,niña)")),xlim=range(nus),nclass=30,prob=TRUE)
abline(v=mean(nus[fseas=="niña"]),lty=2,lwd=2)
hist(xis[fseas=="niña"],xlab=expression(xi),main=expression(paste(pi,"(",xi," |y,niña)")),xlim=c(0,5),nclass=30,prob=TRUE)
abline(v=mean(xis[fseas=="niña"]),lty=2,lwd=2)
hist(nus[fseas=="niño"],xlab=expression(nu),main=expression(paste(pi,"(",nu," |y,niño)")),xlim=range(nus),nclass=30,prob=TRUE)
abline(v=mean(nus[fseas=="niño"]),lty=2,lwd=2)
hist(xis[fseas=="niño"],xlab=expression(xi),main=expression(paste(pi,"(",xi," |y,niño)")),xlim=c(0,5),nclass=30,prob=TRUE)
abline(v=mean(xis[fseas=="niño"]),lty=2,lwd=2)
hist(nus[fseas=="neutral"],xlab=expression(nu),main=expression(paste(pi,"(",nu," |y,neutral)")),xlim=range(nus),nclass=30,prob=TRUE)
abline(v=mean(nus[fseas=="neutral"]),lty=2,lwd=2)
hist(xis[fseas=="neutral"],xlab=expression(xi),main=expression(paste(pi,"(",xi," |y,neutral)")),xlim=c(0,5),nclass=30,prob=TRUE)
abline(v=mean(xis[fseas=="neutral"]),lty=2,lwd=2)

# (PLOT) Posterior Distribution of Lambda Given ENSO and Year
res.lambda=NULL
for(ti in 1:71){
  dm=ws[ti,]
  res.lambda=rbind(res.lambda,cbind(rep(1924+ti,nrow(theta.pois)),c((dm%*%t(theta.pois))^2)))}
enso=ws[,1:3]%*%c(1,1,2)
year=as.factor(res.lambda[,1])
upp.lambda=by(res.lambda[,2],year,function(x) quantile(x,p=0.95))
med.lambda=by(res.lambda[,2],year,function(x) quantile(x,p=0.5))
low.lambda=by(res.lambda[,2],year,function(x) quantile(x,p=0.05))

par(mfrow=c(1,1))
plot(1925:1995,med.lambda,type="b",ylim=c(min(low.lambda),max(upp.lambda)),main="lambda",xlab="Year",ylab="lambda",pch=19+as.numeric(enso),lwd=2)
lines(1925:1995,upp.lambda,ylim=c(min(low.lambda),max(upp.lambda)),lty=2,lwd=1)
lines(1925:1995,low.lambda,ylim=c(min(low.lambda),max(upp.lambda)),lty=2,lwd=1)
legend(1930,4.2,c("niña","niño","neutral"),pch=c(20,21,22),cex=1.2,horiz=TRUE)




# (PLOT) Posterior Distribution for the Mean Return Level
upp.mrl1=med.mrl1=low.mrl1=rep(NA,71)
upp.mrl70=med.mrl70=low.mrl70=upp.mrl50=med.mrl50=low.mrl50=rep(NA,71)
for(i in 1:71){
	if(indi.ninya[i]==TRUE){
		#For 1 year
		upp.mrl1[i]=quantile(mrl1.enso[,1,i],p=0.95)
		med.mrl1[i]=quantile(mrl1.enso[,1,i],p=0.5)
		low.mrl1[i]=quantile(mrl1.enso[,1,i],p=0.05)
	}
	if(indi.ninyo[i]==TRUE){
		#For 1 Year
		upp.mrl1[i]=quantile(mrl1.enso[,2,i],p=0.95)
		med.mrl1[i]=quantile(mrl1.enso[,2,i],p=0.5)
		low.mrl1[i]=quantile(mrl1.enso[,2,i],p=0.05)
	}
	if(indi.neutral[i]==TRUE){
		upp.mrl1[i]=quantile(mrl1.enso[,3,i],p=0.95)
		med.mrl1[i]=quantile(mrl1.enso[,3,i],p=0.5)
		low.mrl1[i]=quantile(mrl1.enso[,3,i],p=0.05)
		}
}	

mrlsim=c(mrl1.enso[,1,indi.ninya],
		 mrl1.enso[,2,indi.ninyo],
		 mrl1.enso[,3,indi.neutral])
mrlsimc=factor(c(rep("Niña (1Y)",nrep*sum(indi.ninya)),
				rep("Niño (1Y)",nrep*sum(indi.ninyo)),
				rep("Neutral (1Y)",nrep*sum(indi.neutral))),
				levels=c("Niña (1Y)","Niño (1Y)","Neutral (1Y)"))
enso=ws[,1:3]%*%c(1,1,2)


par(mfrow=c(2,1))
boxplot(mrlsim ~ mrlsimc,xlab="",ylab="Billions of 1995 US$",main=expression(paste(pi,"(",L[1]," |y,ENSO)")),outline=FALSE,cex.axis=.8, cex.lab=1.2,cex.axis=1.2,cex.main=1.2)
plot(1925:1995,med.mrl1,type="b",ylim=c(min(low.mrl1),max(upp.mrl1)),main=expression(paste(pi,"(",L[1]," |y,ENSO)")),xlab="Year",ylab="mrl",pch=19+as.numeric(enso),lwd=2)
lines(1925:1995,upp.mrl1,lty=2,lwd=1)
lines(1925:1995,low.mrl1,lty=2,lwd=1)
legend(1925,3.3,c("niña","niño","neutral"),pch=c(20,21,22),cex=1.2,horiz=TRUE)

 


# DIC Calculation
# Posterior log-likelihood:
##Only for nu and xi:
ll=apply(theta,1,function(x) kp(x))

Dbar=mean(-2*ll)

theta.mean=apply(theta,2,mean)

Dhat=-2*kp(theta.mean)

pD=Dbar-Dhat
DIC=Dbar+pD
DIC

##DIC calculation for Poisson model
ll.pois=apply(theta.pois,1,function(x) l.poisson(x,ws=ws,deltai=deltai))

Dbar.pois=mean(-2*ll.pois)
theta.pois.mean=apply(theta.pois,2,mean)
Dhat.pois=-2*l.poisson(theta.pois.mean,ws=ws,deltai=deltai)
pD.pois=Dbar.pois-Dhat.pois
DIC.pois=Dbar.pois+pD.pois
DIC.pois


#Model b:
#Dbar: 92
#pD: 3
#DIC: 95

#Model c:
#Dbar: 89
#pD: 3
#DIC: 92

#Model d:
#Dbar: 86
#pD: 5
#DIC: 91

#Model e:
#Dbar: 86
#pD: 11
#DIC: 97


