### This is the commented R Code used to perform 
### the analysis of San Juan Data in Cabras, Castellanos and Gamerman "A default Bayesian approach for regression on extremes".
rm(list=ls(all=TRUE))

#setwd("sanjuan/")

library(mnormt)
library(foreign)
library(MASS)
library(POT)

set.seed(17)

# Simulations from a Generalized Pareto Distributions
# in xi, sigma parameterization
pareto.simulations<-function(n, xi,  sigma,  u=0){
	res=rep(NA,n)
	exp.case=abs(xi)<1e-10
	nexp.case=!exp.case
    res[exp.case]=rexp(sum(exp.case),1/sigma[exp.case])
    res[nexp.case]=u+(sigma[nexp.case]/xi[nexp.case])*((1-runif(sum(nexp.case)))^(-xi[nexp.case])-1)
    return(res)
}

# Change of parameterization from xi.nu to xi.sigma
# return xi.sigma parameterization
xi.nu.to.xi.sigma=function(par) cbind(par[,1],par[,2]/(1+par[,1]))

# Simulation from a GPD process with z,x covariates
sim.gpd.cov=function(n,beta,gamma,z,x){
	xi.lp=z%*%gamma # Linear predictor for xi
	nu.lp=x%*%beta # Linear predictor for nu
	nu=exp(nu.lp)
	xi=exp(xi.lp)-1
	xi.sigma= xi.nu.to.xi.sigma(cbind(xi,nu))
	y=pareto.simulations(n,xi.sigma[,1],xi.sigma[,2])
	return(y)
}

data=read.table(file="sanjuan-daily-m.csv",sep=";",dec=",",header=TRUE)

# threshold in mm, that can differ for each season:
u.dry=u.wet=40

## We clusterize extremes and take the maximum in each cluster 
tiempo=as.numeric(as.Date(paste(data$Year,data$Month,data$Day,sep="-")))
tiempo=tiempo-min(tiempo)
datos=cbind(data$Rainfall,tiempo)
colnames(datos)=c("obs","time")
clust.datos=clust(datos,u=30,tim.cond=3,clust.max=TRUE,plot=FALSE)

data=data[clust.datos[,3],]
##Season definition
season=(data$Month%in%c(1:7))
season[season]="dry" # x=0
season[season=="FALSE"]="wet" # x=1
season=factor(season)
n=dim(data)[1] # Salmple size
selec=rep(FALSE,n)
for(i in 1:n) if((season[i]=="dry" && data$Rainfall[i]>u.dry) || (season[i]=="wet" && data$Rainfall[i]>u.wet)) selec[i]=TRUE 
# Here goes the definition of the final DataSet
data=data[selec,]
y=data$Rainfall
# Final Season Vector
season=(data$Month%in%c(1:7))
season[season]="dry" # x=0
season[season=="FALSE"]="wet" # x=1
season=factor(season)
table(season)

# (PLOT) Data
time.rain=(data$Year-1967)*365+data$Day.year
plot(time.rain,y,type="p",main="San Juan rainfall levels",xlab="Time period (days from 1967 to 2001)",ylab="mm",pch=19+as.numeric(season),col=1, cex.lab=1.2,cex.axis=1.2,cex.main=1.2 )
legend(1000,210,c("dry season","wet season"),pch=c(20,21),cex=1.2)

## Definition of Exceedences by Season
y[season=="dry"]=y[season=="dry"]-u.dry
y[season=="wet"]=y[season=="wet"]-u.wet


### Definition of desing matrices x,z and w for different models ###

##Design Matrix for Model 1: nu and xi are the same for both seasons:
#mylm=lm(y~1,x=TRUE)
#x=z=w=mylm$x
#ws=unique(w)
#colnames(x)=colnames(z)=colnames(w)=c("I","wet")

##Design Matrix for Model 2: nu is different in each season and xi is the same for both seasons:
#mylm=lm(y~season,x=TRUE)
#x=mylm$x
#ws=unique(w)
#mylm=lm(y~1,x=TRUE)
#z=w=mylm$x
#colnames(x)=colnames(z)=colnames(w)=c("I","wet")

##Design Matrix for Model 3: xi is different in each season and nu is the same for both seasons:
mylm=lm(y~season,x=TRUE)
z=w=mylm$x
mylm=lm(y~1,x=TRUE)
x=mylm$x
ws=unique(w)
colnames(x)="I"
colnames(z)=colnames(w)=colnames(ws)=c("I","wet")


# Number of parameters
p=ncol(x)
q=ncol(z)
r=ncol(w)
n=nrow(x)
p
q
r
n



# log posterior kernel
l.zx=function(y,z,x,beta,gamma,n){
	xi=exp(z%*%gamma)-1
    if(any(xi<(-0.5))) return(-Inf)
	nu=exp(x%*%beta)
	if(any((1+y*(xi^2+xi)/nu)<=0)) return(-Inf)
	res=rep(-Inf,n)
	exp.case=abs(xi)<1e-10
	nexp.case=!exp.case
	res[exp.case]=-log(nu[exp.case])-y[exp.case]/nu[exp.case]
	res[nexp.case]=log((1+xi[nexp.case]))-log(nu[nexp.case])-(1/xi[nexp.case]+1)*log(1+y[nexp.case]*(xi[nexp.case]^2+xi[nexp.case])/nu[nexp.case])
	return(sum(res))
	}
kp=function(theta) l.zx(y,z,x,beta=theta[1:p],gamma=theta[(p+1):(p+q)],n)

# Posterior mode
init.point=rep(0,p+q)
init.point[1]=3
tmp=optim(par=init.point,kp,control=list(fnscale=-1,maxit=10000),hessian=TRUE,method=c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN")[1])
tmp$convergence
mode=c(tmp$par)
mode


# Setup of MCMC for the GPD part of posterior
nrep=20000
burn.in=10000
varlabel=c(paste("beta[",1:p,"]",sep=""),paste("gamma[",1:q,"]",sep=""))
theta=array(NA,dim=c(burn.in+nrep,p+q))
theta.star=rep(NA,p+q)
colnames(theta)=varlabel

# Standard Deviation of the Jumping distribution for different models
#Model 1:
#sigma.jump=c(0.3,0.25)
#Model 2:
#sigma.jump=c(0.3,0.3,0.25)
#Model 3:
#sigma.jump=c(0.3,0.25,0.25)

# Automatically one can use, as initial values for the variance
# of proposal, the diagonal elements of the inverse of Hessian 
# matrix of log-posterior kernel, calculated at posterior mode
# in order to reach a reasonable acceptance rate we multiply such elements by a factor k (here k is 2).
sigma.jump=sqrt(abs(diag(solve(tmp$hessian))))*2

# The MCMC and the Acceptance Rate (acc)
theta[1,]=mode
acc=rep(0,p+q)
for(i in 2:(burn.in+nrep)){
	sampord=sample(p+q,size=p+q)
	theta.star=theta[i-1,]
	for(j in sampord){
		theta.star[j]=rnorm(1,mean=theta[i-1,j],sd=sigma.jump[j])
	    lratio=kp(theta.star)-kp(theta[i-1,])
	    if(log(runif(1))>lratio) theta.star[j]= theta[i-1,j]
		}
		theta[i,]=theta.star
		acc=acc+(theta[i,]!=theta[i-1,])
		cat("i=",i,"Acceptance Rate=",round(acc/i,1)*10,"\n")
	}
theta=theta[-(1:burn.in),]

# MCMC diagnostic
par(mfrow=c(3,3))
for(j in 1:(p+q)){
	plot.ts(theta[,j],main=varlabel[j])
  acf(theta[,j],main=varlabel[j])
	hist(theta[,j],main=varlabel[j])
}


##Counting part of the Model ###
period=(2001-1967)
##Each year has 5 wet months and 7 dry monts. 
deltai=c(period*5,period*7)/12

# LOG-Likelihood for Poisson Process 
l.poisson=function(alpha,w,deltai=deltai,ws) sum(log((w%*%alpha)^2))-sum(deltai*(ws%*%alpha)^2)
	
# Posterior MODE
init.point=rep(2,r)
tmp=optim(par=init.point,l.poisson,control=list(fnscale=-1,maxit=10000),hessian=TRUE,method=c("Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "SANN")[1],w=w,deltai=deltai,ws=ws)
tmp$convergence
mode.pois=c(tmp$par)
mode.pois


# Setup of MCMC for the GPD part of posterior
# Standard Deviation of Proposal Distribution
# Automatically one can use, as initial values for the variance
# of proposal, the diagonal elements of the inverse of Hessian 
# matrix of log-posterior kernel, calculated at posterior mode
# in order to reach a reasonable acceptance rate we multiply such elements by a factor k (here k is 2).
sigma.jump=sqrt(abs(diag(solve(tmp$hessian))))*2

#sigma.jump=c(0.04,0.07)*6


varlabel=paste("alpha[",1:r,"]",sep="")
theta.pois=array(NA,dim=c(burn.in+nrep,r))
theta.star.pois=rep(NA,r)
colnames(theta.pois)=varlabel
theta.pois[1,]=mode.pois
acc=rep(0,r)


# The MCMC and the Acceptance Rate (acc)
for(i in 2:(burn.in+nrep)){
	sampord=sample(r,size=r)
	theta.star.pois=theta.pois[i-1,]
	for(j in sampord){
		theta.star.pois[j]=rnorm(1,mean=theta.pois[i-1,j],sd=sigma.jump[j])
	    lratio=l.poisson(alpha=theta.star.pois,w=w,deltai,ws=ws)-l.poisson(alpha=theta.pois[i-1,],w=w,deltai,ws=ws)
	    if(log(runif(1))>lratio) theta.star.pois[j]= theta.pois[i-1,j]
		}
		theta.pois[i,]=theta.star.pois
		acc=acc+(theta.pois[i,]!=theta.pois[i-1,])
		cat("i=",i,"Acceptance Rate=",round(acc/i,1)*10,"\n")
	}

theta.pois=theta.pois[-(1:burn.in),]

# MCMC Diagnostic
par(mfrow=c(2,3))
for(j in 1:r){
	plot.ts(theta.pois[,j],main=varlabel[j])
    acf(theta.pois[,j],main=varlabel[j])
	hist(theta.pois[,j],main=varlabel[j])
}

######################## Analyses of the Joint Posterior Distribution ########################
# Posterior Predictive distribution of NU, XI, Lambda and Mean Return Level (mrl) for N-years (N=50,100,200) given Season

##nu does not depend on season
nu=c(exp(1%*%t(theta[,1:p])))

##xi and lambda given dry season
dm=c(1,0)
xi.dry=c(exp(dm%*%t(theta[,(p+1):(p+q)]))-1)
lambda.dry=c((dm%*%t(theta.pois))^2)
mrl50.dry=c(u.dry+(nu/(xi.dry*(1+xi.dry)))*((lambda.dry*50*7/12)^xi.dry-1))
mrl100.dry=c(u.dry+(nu/(xi.dry*(1+xi.dry)))*((lambda.dry*100*7/12)^xi.dry-1))
mrl200.dry=c(u.dry+(nu/(xi.dry*(1+xi.dry)))*((lambda.dry*200*7/12)^xi.dry-1))
q50.dry=quantile(mrl50.dry,p=c(0.025,0.975))
q100.dry=quantile(mrl100.dry,p=c(0.025,0.975))
q200.dry=quantile(mrl200.dry,p=c(0.025,0.975))
mean(mrl50.dry)
q50.dry
mean(mrl100.dry)
q100.dry
mean(mrl200.dry)
q200.dry

##xi and lambda given wet season
dm=c(1,1)
xi.wet=c(exp(dm%*%t(theta[,(p+1):(p+q)]))-1)
lambda.wet=c((dm%*%t(theta.pois))^2)
mrl50.wet=c(u.wet+(nu/(xi.wet*(1+xi.wet)))*((lambda.wet*50*5/12)^xi.wet-1))
mrl100.wet=c(u.wet+(nu/(xi.wet*(1+xi.wet)))*((lambda.wet*100*5/12)^xi.wet-1))
mrl200.wet=c(u.wet+(nu/(xi.wet*(1+xi.wet)))*((lambda.wet*200*5/12)^xi.wet-1))
q50.wet=quantile(mrl50.wet,p=c(0.025,0.975))
q100.wet=quantile(mrl100.wet,p=c(0.025,0.975))
q200.wet=quantile(mrl200.wet,p=c(0.025,0.975))
mean(mrl50.wet)
q50.wet
mean(mrl100.wet)
q100.wet
mean(mrl200.wet)
q200.wet


#Predictive posterior probabilities for a Large rain
#A rain of 558 mm (22 inches) ocurred on 6-7 October 1985, in Barrio Mameyes
mean(mrl50.wet>558)
mean(mrl100.wet>558)
mean(mrl200.wet>558)
mean(mrl50.dry>558)
mean(mrl100.dry>558)
mean(mrl200.dry>558)

## (PLOT) of posterior distributions
#Histograms for NU, XI and Lambda:
layout(matrix(c(1,1,2,3,4,5), 3, 2, byrow=TRUE),respect=TRUE)
par(mar=c(5.1, 14, 1.4, 12))
hist(nu,xlab=expression(nu),ylab="Density",main=expression(paste(pi,"(",nu," |y)")), cex.lab=1.5,cex.axis=1.5,cex.main=1.5,prob=TRUE)
par(mar=c(5.1, 4.1, 1.4, 2.1))
hist(xi.dry,xlab=expression(xi),ylab="Density",main=expression(paste(pi,"(",xi," |y,dry)")),xlim=range(c(xi.dry,xi.wet)), cex.lab=1.5,cex.axis=1.5,cex.main=1.5,prob=TRUE)
hist(lambda.dry,xlab=expression(lambda),ylab="Density",main=expression(paste(pi,"(",lambda," |y,dry)")), xlim=range(c(lambda.dry,lambda.wet)),cex.lab=1.5,cex.axis=1.5,cex.main=1.5 ,prob=TRUE)
hist(xi.wet,xlab=expression(xi),ylab="Density",main=expression(paste(pi,"(",xi," |y,wet)")), xlim=range(c(xi.dry,xi.wet)), cex.lab=1.5,cex.axis=1.5,cex.main=1.5 ,prob=TRUE)
hist(lambda.wet,xlab=expression(lambda),ylab="Density",main=expression(paste(pi,"(",lambda," |y,wet)")), xlim=range(c(lambda.dry,lambda.wet)), cex.lab=1.5,cex.axis=1.5,cex.main=1.5,prob=TRUE)

#Box Plot for 50 and 100 year Mean Return Level given Season:
mrlsim=c(mrl50.dry,mrl100.dry,mrl50.wet,mrl100.wet)
mrlsimc=factor(c(rep("Dry (50Y)",nrep),
				rep("Dry (100Y)",nrep),
				rep("Wet (50Y)",nrep),
				rep("Wet (100Y)",nrep)),
				levels=c("Dry (50Y)","Dry (100Y)","Wet (50Y)","Wet (100Y)"))
boxplot(mrlsim ~ mrlsimc,xlab="",ylab="mm",main=expression(paste(pi,"(",L[k]," |y,season)")),ylim=c(0,800),outline=FALSE, cex.lab=1.2,cex.axis=1.2,cex.main=1.2  )
abline(h=558,lty=2)

##QQ-plots for assessing the Goodness-of-fit by Season
scale.dry=nu/(1+xi.dry)
y.simul.wet=array(NA,dim=c(nrep,length(y[season=="wet"])))
y.simul.dry=array(NA,dim=c(nrep,length(y[season=="dry"])))
scale.wet=nu/(1+xi.wet)
prob.points.dry=ppoints(y[season=="dry"])
prob.points.wet=ppoints(y[season=="wet"])

for(i in 1:nrep){
	y.simul.dry[i,]=qgpd(prob.points.dry,scale=scale.dry[i],shape=xi.dry[i])
	y.simul.wet[i,]=qgpd(prob.points.wet,scale=scale.wet[i],shape=xi.wet[i])}

quantile.y.dry=apply(y.simul.dry,2,median)
quantile.y.wet=apply(y.simul.wet,2,median)


par(mfrow=c(1,2))
plot(sort(y[season=="dry"]),quantile.y.dry,main="QQ-plot for dry season",xlab="Observed Quantiles",
       ylab="Theorethical Quantiles",cex.lab=1.2,cex.axis=1.2,cex.main=1.2)
abline(0,1)
plot(sort(y[season=="wet"]),quantile.y.wet,main="QQ-plot for wet season",xlab="Observed Quantiles",
       ylab="Theorethical Quantiles",cex.lab=1.2,cex.axis=1.2,cex.main=1.2)
abline(0,1)


# DIC & BIC Calculation
# Posterior log-likelihood:
ll=apply(theta,1,function(x) kp(x))
Dbar=mean(-2*ll)
theta.mean=apply(theta,2,mean)
Dhat=-2*kp(theta.mean)
pD=Dbar-Dhat
DIC=Dbar+pD
DIC

#Model 1-.
#DIC: 1070
#Dbar: 1068
#pD: 2

#Model 2-.
#DIC: 1072
#Dbar: 1069
#pD: 3

#Model 3-.
#DIC: 1069
#Dbar: 1066
#pD: 3


#Model 4-.
#DIC: 1072
#Dbar: 1068
#pD: 4

##DIC calculation for Poisson model
ll.pois=apply(theta.pois,1,function(x) l.poisson(x,w,deltai=deltai,ws))
Dbar.pois=mean(-2*ll.pois)
theta.pois.mean=apply(theta.pois,2,mean)
Dhat.pois=-2*l.poisson(theta.pois.mean,w,deltai=deltai,ws)
pD.pois=Dbar.pois-Dhat.pois
DIC.pois=Dbar.pois+pD.pois
round(DIC.pois)

##Model 1 for lambda-. Calculated from the posterior distribution, like in Katz:
#Dbar: -71
#pD: 1
#DIC: -73

##Model 2 for lambda-.
#Dbar: -87
#pD: 2
#DIC: -85 

